/* PgSqlClient - ADO.NET Data Provider for PostgreSQL 7.4+
 * Copyright (c) 2003-2004 Carlos Guzman Alvarez
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Data;
using System.Text;
using System.Text.RegularExpressions;

namespace PostgreSql.Data.PgSqlClient.DbSchema
{
	internal class PgTableStatisticsSchema : PgAbstractDbSchema
	{
		#region Constructors

		public PgTableStatisticsSchema() : base("Table_Statistics")
		{
		}

		#endregion

		#region Add Methods

		public override void AddTables()
		{
			AddTable("pg_class");
		}

		public override void AddRestrictionColumns()
		{
			AddRestrictionColumn("pg_namespace.nspname"	, "TABLE_SCHEMA", null);
			AddRestrictionColumn("pg_class.relname"		, "TABLE_NAME", null);
			AddRestrictionColumn("pg_namespace.staattnum", "ORDINAL_POSITION", null);			
		}

		public override void AddDataColumns()
		{
			AddDataColumn("pg_statistic.stanullfrac", "FRACTION");
			AddDataColumn("pg_statistic.stawidth"	, "WIDTH");
			AddDataColumn("pg_statistic.stadistinc"	, "DISTINCT");
			AddDataColumn("pg_statistic.stakind"	, "KIND");
			AddDataColumn("pg_statistic.stanumbers"	, "NUMERICAL_STATISTICS");			
		}

		public override void AddJoins()
		{
			AddJoin("left join", "pg_statistic", "pg_class.oid = Pgstatistic.starelid");
		}

		public override void AddOrderByColumns()
		{
			AddOrderBy("pg_namespace.nspname");
			AddOrderBy("pg_class.relname");
		}

		public override void AddWhereFilters()
		{
			AddWhereFilter("pg_class.relkind = 'r'");
		}

		#endregion

		#region Parse Methods

		public override object[] ParseRestrictions(object[] restrictions)
		{
			object[] parsed = restrictions;

			return parsed;
		}

		#endregion
	}
}